""" Generator functions related to trial and test functions and the accumulation loop"""

from dune.perftool.generation import pymbolic_expr, symbol, globalarg
from dune.perftool.ufl.modified_terminals import ModifiedArgumentDescriptor


@symbol
def name_testfunction(ma):
    if len(ma.argexpr.element().sub_elements()) > 0:
        pass
    return "{}a{}".format("grad_" if ma.grad else "", ma.argexpr.number())


@pymbolic_expr
def pymbolic_testfunction(ma):
    assert bool(ma.index) == ma.grad
    from pymbolic.primitives import Subscript, Variable
    v = Variable(name_testfunction(ma))
    globalarg(name_testfunction(ma))
    if ma.grad:
        from dune.perftool.pdelab import name_index
        return Subscript(v, Variable(name_index(ma.index)))
    else:
        return v


@symbol
def name_trialfunction(ma):
    return "{}c{}".format("grad_" if ma.grad else "", ma.argexpr.count())


@pymbolic_expr
def pymbolic_trialfunction(ma):
    assert bool(ma.index) == ma.grad
    from pymbolic.primitives import Subscript, Variable
    v = Variable(name_trialfunction(ma))
    globalarg(name_trialfunction(ma))
    if ma.grad:
        from dune.perftool.pdelab import name_index
        return Subscript(v, Variable(name_index(ma.index)))
    else:
        return v


@symbol
def name_testfunctionspace(*a):
    # TODO
    return "lfsv"


@symbol
def name_trialfunctionspace(*a):
    # TODO
    return "lfsu"


def name_argumentspace(ma):
    if ma.argexpr.number() == 0:
        return name_testfunctionspace(ma)
    if ma.argexpr.number() == 1:
        return name_trialfunctionspace(ma)
    # We should never encounter an argument other than 0 or 1
    assert False


def name_argument(ma):
    if ma.argexpr.number() == 0:
        return name_testfunction(ma)
    if ma.argexpr.number() == 1:
        return name_trialfunction(ma)
    # We should never encounter an argument other than 0 or 1
    assert False


def pymbolic_argument(ma):
    if ma.argexpr.number() == 0:
        return pymbolic_testfunction(ma)
    if ma.argexpr.number() == 1:
        return pymbolic_trialfunction(ma)
    assert False

@symbol
def name_residual():
    return "r"
